/* abstractlist.vala
 *
 * Copyright (C) 2007  Jürg Billeter
 * Copyright (C) 2009  Didier Villevalois
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes' Villevalois <ptitjes@free.fr>
 */

#include <glib.h>
#include <glib-object.h>


#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))

typedef struct _GeeIterable GeeIterable;
typedef struct _GeeIterableIface GeeIterableIface;

#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))

typedef struct _GeeIterator GeeIterator;
typedef struct _GeeIteratorIface GeeIteratorIface;

#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))

typedef struct _GeeCollection GeeCollection;
typedef struct _GeeCollectionIface GeeCollectionIface;

#define GEE_TYPE_ABSTRACT_COLLECTION (gee_abstract_collection_get_type ())
#define GEE_ABSTRACT_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollection))
#define GEE_ABSTRACT_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollectionClass))
#define GEE_IS_ABSTRACT_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_COLLECTION))
#define GEE_IS_ABSTRACT_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_COLLECTION))
#define GEE_ABSTRACT_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollectionClass))

typedef struct _GeeAbstractCollection GeeAbstractCollection;
typedef struct _GeeAbstractCollectionClass GeeAbstractCollectionClass;
typedef struct _GeeAbstractCollectionPrivate GeeAbstractCollectionPrivate;

#define GEE_TYPE_LIST (gee_list_get_type ())
#define GEE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_LIST, GeeList))
#define GEE_IS_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_LIST))
#define GEE_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_LIST, GeeListIface))

typedef struct _GeeList GeeList;
typedef struct _GeeListIface GeeListIface;

#define GEE_TYPE_ABSTRACT_LIST (gee_abstract_list_get_type ())
#define GEE_ABSTRACT_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_LIST, GeeAbstractList))
#define GEE_ABSTRACT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_LIST, GeeAbstractListClass))
#define GEE_IS_ABSTRACT_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_LIST))
#define GEE_IS_ABSTRACT_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_LIST))
#define GEE_ABSTRACT_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_LIST, GeeAbstractListClass))

typedef struct _GeeAbstractList GeeAbstractList;
typedef struct _GeeAbstractListClass GeeAbstractListClass;
typedef struct _GeeAbstractListPrivate GeeAbstractListPrivate;

/**
 * Implemented by classes that support a simple iteration over instances of the
 * collection.
 */
struct _GeeIteratorIface {
	GTypeInterface parent_iface;
	gboolean (*next) (GeeIterator* self);
	gpointer (*get) (GeeIterator* self);
};

/**
 * Implemented by classes that support a simple iteration over instances of the
 * collection.
 */
struct _GeeIterableIface {
	GTypeInterface parent_iface;
	GeeIterator* (*iterator) (GeeIterable* self);
	GType (*get_element_type) (GeeIterable* self);
};

/**
 * Serves as the base interface for implementing collection classes. Defines
 * size, iteration, and modification methods.
 */
struct _GeeCollectionIface {
	GTypeInterface parent_iface;
	gboolean (*contains) (GeeCollection* self, gconstpointer item);
	gboolean (*add) (GeeCollection* self, gconstpointer item);
	gboolean (*remove) (GeeCollection* self, gconstpointer item);
	void (*clear) (GeeCollection* self);
	gboolean (*add_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*contains_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*remove_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*retain_all) (GeeCollection* self, GeeCollection* collection);
	gpointer* (*to_array) (GeeCollection* self, int* result_length1);
	gint (*get_size) (GeeCollection* self);
	gboolean (*get_is_empty) (GeeCollection* self);
};

/**
 * Serves as the base class for implementing collection classes.
 */
struct _GeeAbstractCollection {
	GObject parent_instance;
	GeeAbstractCollectionPrivate * priv;
};

struct _GeeAbstractCollectionClass {
	GObjectClass parent_class;
	gboolean (*contains) (GeeAbstractCollection* self, gconstpointer item);
	gboolean (*add) (GeeAbstractCollection* self, gconstpointer item);
	gboolean (*remove) (GeeAbstractCollection* self, gconstpointer item);
	void (*clear) (GeeAbstractCollection* self);
	gpointer* (*to_array) (GeeAbstractCollection* self, int* result_length1);
	gboolean (*add_all) (GeeAbstractCollection* self, GeeCollection* collection);
	gboolean (*contains_all) (GeeAbstractCollection* self, GeeCollection* collection);
	gboolean (*remove_all) (GeeAbstractCollection* self, GeeCollection* collection);
	gboolean (*retain_all) (GeeAbstractCollection* self, GeeCollection* collection);
	GeeIterator* (*iterator) (GeeAbstractCollection* self);
	gint (*get_size) (GeeAbstractCollection* self);
	gboolean (*get_is_empty) (GeeAbstractCollection* self);
};

/**
 * Represents a collection of items in a well-defined order.
 */
struct _GeeListIface {
	GTypeInterface parent_iface;
	gpointer (*get) (GeeList* self, gint index);
	void (*set) (GeeList* self, gint index, gconstpointer item);
	gint (*index_of) (GeeList* self, gconstpointer item);
	void (*insert) (GeeList* self, gint index, gconstpointer item);
	void (*remove_at) (GeeList* self, gint index);
	GeeList* (*slice) (GeeList* self, gint start, gint stop);
	gpointer (*first) (GeeList* self);
	gpointer (*last) (GeeList* self);
	void (*insert_all) (GeeList* self, gint index, GeeCollection* collection);
};

/**
 * Serves as the base class for implementing list classes.
 */
struct _GeeAbstractList {
	GeeAbstractCollection parent_instance;
	GeeAbstractListPrivate * priv;
};

struct _GeeAbstractListClass {
	GeeAbstractCollectionClass parent_class;
	gpointer (*get) (GeeAbstractList* self, gint index);
	void (*set) (GeeAbstractList* self, gint index, gconstpointer item);
	gint (*index_of) (GeeAbstractList* self, gconstpointer item);
	void (*insert) (GeeAbstractList* self, gint index, gconstpointer item);
	void (*remove_at) (GeeAbstractList* self, gint index);
	GeeList* (*slice) (GeeAbstractList* self, gint start, gint stop);
	gpointer (*first) (GeeAbstractList* self);
	gpointer (*last) (GeeAbstractList* self);
	void (*insert_all) (GeeAbstractList* self, gint index, GeeCollection* collection);
};

struct _GeeAbstractListPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};


static gpointer gee_abstract_list_parent_class = NULL;
static GeeListIface* gee_abstract_list_gee_list_parent_iface = NULL;

GType gee_iterator_get_type (void);
GType gee_iterable_get_type (void);
GType gee_collection_get_type (void);
GType gee_abstract_collection_get_type (void);
GType gee_list_get_type (void);
GType gee_abstract_list_get_type (void);
#define GEE_ABSTRACT_LIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_ABSTRACT_LIST, GeeAbstractListPrivate))
enum  {
	GEE_ABSTRACT_LIST_DUMMY_PROPERTY,
	GEE_ABSTRACT_LIST_G_TYPE,
	GEE_ABSTRACT_LIST_G_DUP_FUNC,
	GEE_ABSTRACT_LIST_G_DESTROY_FUNC
};
gpointer gee_abstract_list_get (GeeAbstractList* self, gint index);
static gpointer gee_abstract_list_real_get (GeeAbstractList* self, gint index);
void gee_abstract_list_set (GeeAbstractList* self, gint index, gconstpointer item);
static void gee_abstract_list_real_set (GeeAbstractList* self, gint index, gconstpointer item);
gint gee_abstract_list_index_of (GeeAbstractList* self, gconstpointer item);
static gint gee_abstract_list_real_index_of (GeeAbstractList* self, gconstpointer item);
void gee_abstract_list_insert (GeeAbstractList* self, gint index, gconstpointer item);
static void gee_abstract_list_real_insert (GeeAbstractList* self, gint index, gconstpointer item);
void gee_abstract_list_remove_at (GeeAbstractList* self, gint index);
static void gee_abstract_list_real_remove_at (GeeAbstractList* self, gint index);
GeeList* gee_abstract_list_slice (GeeAbstractList* self, gint start, gint stop);
static GeeList* gee_abstract_list_real_slice (GeeAbstractList* self, gint start, gint stop);
gpointer gee_abstract_list_first (GeeAbstractList* self);
static gpointer gee_abstract_list_real_first (GeeAbstractList* self);
gint gee_collection_get_size (GeeCollection* self);
gpointer gee_abstract_list_last (GeeAbstractList* self);
static gpointer gee_abstract_list_real_last (GeeAbstractList* self);
GeeIterator* gee_iterable_iterator (GeeIterable* self);
gboolean gee_iterator_next (GeeIterator* self);
gpointer gee_iterator_get (GeeIterator* self);
void gee_abstract_list_insert_all (GeeAbstractList* self, gint index, GeeCollection* collection);
static void gee_abstract_list_real_insert_all (GeeAbstractList* self, gint index, GeeCollection* collection);
static void gee_abstract_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void gee_abstract_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



static gpointer gee_abstract_list_real_get (GeeAbstractList* self, gint index) {
	g_return_val_if_fail (self != NULL, NULL);
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_list_get'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


gpointer gee_abstract_list_get (GeeAbstractList* self, gint index) {
	return GEE_ABSTRACT_LIST_GET_CLASS (self)->get (self, index);
}


static void gee_abstract_list_real_set (GeeAbstractList* self, gint index, gconstpointer item) {
	g_return_if_fail (self != NULL);
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_list_set'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return;
}


void gee_abstract_list_set (GeeAbstractList* self, gint index, gconstpointer item) {
	GEE_ABSTRACT_LIST_GET_CLASS (self)->set (self, index, item);
}


static gint gee_abstract_list_real_index_of (GeeAbstractList* self, gconstpointer item) {
	g_return_val_if_fail (self != NULL, 0);
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_list_index_of'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return 0;
}


gint gee_abstract_list_index_of (GeeAbstractList* self, gconstpointer item) {
	return GEE_ABSTRACT_LIST_GET_CLASS (self)->index_of (self, item);
}


static void gee_abstract_list_real_insert (GeeAbstractList* self, gint index, gconstpointer item) {
	g_return_if_fail (self != NULL);
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_list_insert'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return;
}


void gee_abstract_list_insert (GeeAbstractList* self, gint index, gconstpointer item) {
	GEE_ABSTRACT_LIST_GET_CLASS (self)->insert (self, index, item);
}


static void gee_abstract_list_real_remove_at (GeeAbstractList* self, gint index) {
	g_return_if_fail (self != NULL);
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_list_remove_at'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return;
}


void gee_abstract_list_remove_at (GeeAbstractList* self, gint index) {
	GEE_ABSTRACT_LIST_GET_CLASS (self)->remove_at (self, index);
}


static GeeList* gee_abstract_list_real_slice (GeeAbstractList* self, gint start, gint stop) {
	g_return_val_if_fail (self != NULL, NULL);
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_list_slice'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


GeeList* gee_abstract_list_slice (GeeAbstractList* self, gint start, gint stop) {
	return GEE_ABSTRACT_LIST_GET_CLASS (self)->slice (self, start, stop);
}


static gpointer gee_abstract_list_real_first (GeeAbstractList* self) {
	gpointer result;
	g_return_val_if_fail (self != NULL, NULL);
	result = gee_abstract_list_get (self, 0);
	return result;
}


gpointer gee_abstract_list_first (GeeAbstractList* self) {
	return GEE_ABSTRACT_LIST_GET_CLASS (self)->first (self);
}


static gpointer gee_abstract_list_real_last (GeeAbstractList* self) {
	gpointer result;
	g_return_val_if_fail (self != NULL, NULL);
	result = gee_abstract_list_get (self, gee_collection_get_size ((GeeCollection*) self) - 1);
	return result;
}


gpointer gee_abstract_list_last (GeeAbstractList* self) {
	return GEE_ABSTRACT_LIST_GET_CLASS (self)->last (self);
}


static void gee_abstract_list_real_insert_all (GeeAbstractList* self, gint index, GeeCollection* collection) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (collection != NULL);
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator ((GeeIterable*) collection);
		while (TRUE) {
			gpointer item;
			if (!gee_iterator_next (_item_it)) {
				break;
			}
			item = gee_iterator_get (_item_it);
			gee_abstract_list_insert (self, index, item);
			index++;
			((item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (item = (self->priv->g_destroy_func (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (g_object_unref (_item_it), NULL));
	}
}


void gee_abstract_list_insert_all (GeeAbstractList* self, gint index, GeeCollection* collection) {
	GEE_ABSTRACT_LIST_GET_CLASS (self)->insert_all (self, index, collection);
}


static void gee_abstract_list_class_init (GeeAbstractListClass * klass) {
	gee_abstract_list_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeAbstractListPrivate));
	GEE_ABSTRACT_LIST_CLASS (klass)->get = gee_abstract_list_real_get;
	GEE_ABSTRACT_LIST_CLASS (klass)->set = gee_abstract_list_real_set;
	GEE_ABSTRACT_LIST_CLASS (klass)->index_of = gee_abstract_list_real_index_of;
	GEE_ABSTRACT_LIST_CLASS (klass)->insert = gee_abstract_list_real_insert;
	GEE_ABSTRACT_LIST_CLASS (klass)->remove_at = gee_abstract_list_real_remove_at;
	GEE_ABSTRACT_LIST_CLASS (klass)->slice = gee_abstract_list_real_slice;
	GEE_ABSTRACT_LIST_CLASS (klass)->first = gee_abstract_list_real_first;
	GEE_ABSTRACT_LIST_CLASS (klass)->last = gee_abstract_list_real_last;
	GEE_ABSTRACT_LIST_CLASS (klass)->insert_all = gee_abstract_list_real_insert_all;
	G_OBJECT_CLASS (klass)->get_property = gee_abstract_list_get_property;
	G_OBJECT_CLASS (klass)->set_property = gee_abstract_list_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_LIST_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_LIST_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_LIST_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void gee_abstract_list_gee_list_interface_init (GeeListIface * iface) {
	gee_abstract_list_gee_list_parent_iface = g_type_interface_peek_parent (iface);
	iface->get = (gpointer (*)(GeeList* ,gint)) gee_abstract_list_get;
	iface->set = (void (*)(GeeList* ,gint ,gconstpointer)) gee_abstract_list_set;
	iface->index_of = (gint (*)(GeeList* ,gconstpointer)) gee_abstract_list_index_of;
	iface->insert = (void (*)(GeeList* ,gint ,gconstpointer)) gee_abstract_list_insert;
	iface->remove_at = (void (*)(GeeList* ,gint)) gee_abstract_list_remove_at;
	iface->slice = (GeeList* (*)(GeeList* ,gint ,gint)) gee_abstract_list_slice;
	iface->first = (gpointer (*)(GeeList*)) gee_abstract_list_first;
	iface->last = (gpointer (*)(GeeList*)) gee_abstract_list_last;
	iface->insert_all = (void (*)(GeeList* ,gint ,GeeCollection*)) gee_abstract_list_insert_all;
}


static void gee_abstract_list_instance_init (GeeAbstractList * self) {
	self->priv = GEE_ABSTRACT_LIST_GET_PRIVATE (self);
}


GType gee_abstract_list_get_type (void) {
	static GType gee_abstract_list_type_id = 0;
	if (gee_abstract_list_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeAbstractListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_abstract_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeAbstractList), 0, (GInstanceInitFunc) gee_abstract_list_instance_init, NULL };
		static const GInterfaceInfo gee_list_info = { (GInterfaceInitFunc) gee_abstract_list_gee_list_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		gee_abstract_list_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_COLLECTION, "GeeAbstractList", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		g_type_add_interface_static (gee_abstract_list_type_id, GEE_TYPE_LIST, &gee_list_info);
	}
	return gee_abstract_list_type_id;
}


static void gee_abstract_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeAbstractList * self;
	gpointer boxed;
	self = GEE_ABSTRACT_LIST (object);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_abstract_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeAbstractList * self;
	self = GEE_ABSTRACT_LIST (object);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
		case GEE_ABSTRACT_LIST_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_ABSTRACT_LIST_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_ABSTRACT_LIST_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
	}
}




